﻿using System;
using System.Collections;
using System.Windows.Forms;
using JMS.DVB.EPG.Descriptors;

namespace SatIp.Guide
{
    public class EpgEntry : ListViewItem
    {
        public readonly object[] CompareData;
        public String ServiceId
        {
            get ;
            set;
        }
        public String Name
        {
            get;
            set;
        }
        public String Description
        {
            get;
            set;
        }
        public DateTime Start
        {
            get;
            set;
        }
        public DateTime Stop
        {
            get;
            set;
        }
        public EpgEntry()
        {}
        public EpgEntry(ushort service, string name, string description, DateTime start, TimeSpan duration)
        {
            ServiceId = service.ToString();
            Name = name;
            Description = description;
            Start = start;
            Stop = start.Add(duration);
            base.Text = string.Format("{0} (0x{0:x})", service);
            base.SubItems.Add(start.ToString());
            base.SubItems.Add(start.Add(duration).TimeOfDay.ToString());
            base.SubItems.Add(name);
            base.SubItems.Add(description);
            this.CompareData = new object[] { service, start, start.Add(duration), name };
        }

        public class Comparer : IComparer
        {
            public int Compare(object x, object y)
            {
                EpgEntry entry = x as EpgEntry;
                EpgEntry entry2 = y as EpgEntry;
                if (entry2 == null)
                {
                    if (entry == null)
                    {
                        return 0;
                    }
                    return -1;
                }
                if (entry == null)
                {
                    return 1;
                }
                MainForm parent = (MainForm)entry.ListView.Parent;
                int sortIndex = parent.SortIndex;
                bool flag = sortIndex > 0;
                if (!flag)
                {
                    sortIndex = -sortIndex;
                }
                IComparable comparable = (IComparable)entry.CompareData[--sortIndex];
                object obj2 = entry2.CompareData[sortIndex];
                if (obj2 == null)
                {
                    if (comparable == null)
                    {
                        return 0;
                    }
                    return -1;
                }
                if (comparable == null)
                {
                    return 1;
                }
                int num2 = comparable.CompareTo(obj2);
                if (flag)
                {
                    return num2;
                }
                return -num2;
            }
        }
    }
}
