﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;

namespace SatIp.Guide.Rtp
{
    public class RtpClient
    {
        private bool _shouldReceive = false;
        private Socket _socket;
        private EndPoint _endPoint;

        private Thread _receiveThread;
        private string _schema;
        public byte[] Payload { get; set; }

        public RtpClient()
        {
            
        }
        
        public bool Listen
        {
            get { return _shouldReceive; }
            set { _shouldReceive = value; }
        }
        /// <summary>
        /// Start the Listen Thread
        /// </summary>
        /// <param name="address"></param>
        /// <param name="port"></param>
        public void Start(Uri uri)
        {
            _shouldReceive = true;
            _schema = uri.Scheme;
            var ipEndPoint = new IPEndPoint(IPAddress.Parse(uri.Host), uri.Port);
            var endPoint = (EndPoint)ipEndPoint;
            _socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
            _socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, 1);
            _socket.Bind(endPoint);
            var sender = new IPEndPoint(IPAddress.Parse(uri.Host), uri.Port);
            _endPoint = (EndPoint)sender;
            _shouldReceive = true;
            _receiveThread = new Thread(ReceiveCallBack);
            _receiveThread.Start();
        }

        private void ReceiveCallBack()
        {
            byte[] receivedBytes = new byte[1370];
            while (_shouldReceive)
            {
                var receivedByteCount = _socket.ReceiveFrom(receivedBytes, ref _endPoint);
                ProcessPacket(receivedBytes, receivedByteCount);
            }

            _socket.Close();
        }

        private void ProcessPacket(byte[] data,int length)
        {
            
            switch (_schema)
            {
                case "udp" :
                    break;
                case "rtp":
                    var packet = new RtpPacket(data, length);
                    if (Payload== null)
                    {
                        Payload = new byte[packet.PayloadBytesBuffer.Length];
                        Payload = packet.PayloadBytesBuffer;
                    }
                    else
                    {
                        byte[] more = new byte[Payload.Length];
                        more = Payload;
                        Payload = AppendTwoByteArrays(more, packet.PayloadBytesBuffer);
                    }

                     
                    
                    break;
            }
            
        }

        /// <summary>
        /// Stop the Listen Thread 
        /// </summary>
        public void Stop()
        {
            _shouldReceive = false;
        }
        static byte[] AppendTwoByteArrays(byte[] arrayA, byte[] arrayB)
        {
            byte[] outputBytes = new byte[arrayA.Length + arrayB.Length];
            Buffer.BlockCopy(arrayA, 0, outputBytes, 0, arrayA.Length);
            Buffer.BlockCopy(arrayB, 0, outputBytes, arrayA.Length, arrayB.Length);
            return outputBytes;
        }
    }   

}
