﻿using System;
using System.IO;
using System.Linq;


namespace SatIp.Guide.Rtp
{
    public class RtpPacket 
    {
        private static int MinHeaderLength = 12;
        
        // <summary>
		/// Konstruktor
		/// </summary>
		public RtpPacket()
		{

		}
		/// <summary>
		/// Konstuktor
		/// </summary>
		/// <param name="_data"></param>
        public RtpPacket(byte[] data, int length)
		{
		    
            Parse(data, length);
		}

        public Int32 HeaderSize = MinHeaderLength;
        public Int32 Version { get; set; }
        public Boolean Padding { get; set; }
        public Boolean Extension { get; set; }
        public Int32 CsrcCount { get; set; }
        public Boolean Marker { get; set; }
        public Int32 PayloadType { get; set; }
        public UInt16 SequenceNumber { get; set; }
        public UInt32 Timestamp { get; set; }
        public UInt32 SourceId { get; set; }
        //int csrcid = GetRTPHeaderValue(packet, 96, 96);
        public UInt16 ExtensionHeaderId = 0;
        public UInt16 ExtensionLengthAsCount = 0;
        public Int32 ExtensionLengthInBytes = 0;
        public Byte[] PayloadBytesBuffer { get; set; }

        

        /// <summary>
        /// Parse
        /// </summary>
        /// <param name="linearData"></param>
        /// 
        private void Parse(Byte[] data, int length)
        {
            if (data.Length >= MinHeaderLength)
            {
                PacketSize = length;
                Version = ValueFromByte(data[0], 6, 2);
                Padding = Convert.ToBoolean(ValueFromByte(data[0], 5, 1));
                Extension = Convert.ToBoolean(ValueFromByte(data[0], 4, 1));
                CsrcCount = ValueFromByte(data[0], 0, 4);
                Marker = Convert.ToBoolean(ValueFromByte(data[1], 7, 1));
                PayloadType = ValueFromByte(data[1], 0, 7);
                HeaderSize = MinHeaderLength + (CsrcCount * 4);

                //Sequence Nummer
                Byte[] seqNum = new Byte[2];
                seqNum[0] = data[3];
                seqNum[1] = data[2];
                SequenceNumber = System.BitConverter.ToUInt16(seqNum, 0);

                //TimeStamp
                Byte[] timeStmp = new Byte[4];
                timeStmp[0] = data[7];
                timeStmp[1] = data[6];
                timeStmp[2] = data[5];
                timeStmp[3] = data[4];
                Timestamp = System.BitConverter.ToUInt32(timeStmp, 0);

                //SourceId
                Byte[] srcId = new Byte[4];
                srcId[0] = data[8];
                srcId[1] = data[9];
                srcId[2] = data[10];
                srcId[3] = data[11];
                SourceId = System.BitConverter.ToUInt32(srcId, 0);

                //CC
                Byte[] csrcid = new byte[4];

                //Wenn Extension Header
                if (Extension)
                {
                    //ExtensionHeaderId
                    Byte[] extHeaderId = new Byte[2];
                    extHeaderId[1] = data[HeaderSize + 0];
                    extHeaderId[0] = data[HeaderSize + 1];
                    ExtensionHeaderId = System.BitConverter.ToUInt16(extHeaderId, 0);

                    //ExtensionHeaderLength
                    Byte[] extHeaderLength16 = new Byte[2];
                    extHeaderLength16[1] = data[HeaderSize + 2];
                    extHeaderLength16[0] = data[HeaderSize + 3];
                    ExtensionLengthAsCount = System.BitConverter.ToUInt16(extHeaderLength16.ToArray(), 0);

                    //Header Länge anpassen (Länge mal 4 Bytes bzw. Int32)
                    ExtensionLengthInBytes = ExtensionLengthAsCount * 4;
                    HeaderSize += ExtensionLengthInBytes + 4;
                }

                //Daten kopieren
                //_stream.Write(data, 12, data.Length-12);
                PayloadBytesBuffer = new Byte[length - 12];
                Array.Copy(data, 12, PayloadBytesBuffer, 0, length - 12);
                
            }
        }

        public int PacketSize { get; set; }

        /// <summary>
        /// GetValueFromByte
        /// </summary>
        /// <param name="value"></param>
        /// <param name="startPos"></param>
        /// <param name="length"></param>
        /// <returns></returns>
        private Int32 ValueFromByte(Byte value, int startPos, int length)
        {
            Byte mask = 0;
            //Maske erstellen
            for (int i = 0; i < length; i++)
            {
                mask = (Byte)(mask | 0x1 << startPos + i);
            }

            //Ergebnis
            Byte result = (Byte)((value & mask) >> startPos);
            //Fertig
            return Convert.ToInt32(result);
        }
        /// <summary>
        /// ToBytes
        /// </summary>
        /// <returns></returns>
        public Byte[] ToBytes()
        {
            //Ergebnis
            Byte[] bytes = new Byte[HeaderSize + PayloadBytesBuffer.Length];

            //Byte 0
            bytes[0] = (Byte)(Version << 6);
            bytes[0] |= (Byte)(Convert.ToInt32(Padding) << 5);
            bytes[0] |= (Byte)(Convert.ToInt32(Extension) << 4);
            bytes[0] |= (Byte)(Convert.ToInt32(CsrcCount));

            //Byte 1
            bytes[1] = (Byte)(Convert.ToInt32(Marker) << 7);
            bytes[1] |= (Byte)(Convert.ToInt32(PayloadType));

            //Byte 2 + 3
            Byte[] bytesSequenceNumber = BitConverter.GetBytes(SequenceNumber);
            bytes[2] = bytesSequenceNumber[1];
            bytes[3] = bytesSequenceNumber[0];

            //Byte from 4 to 7
            Byte[] bytesTimeStamp = BitConverter.GetBytes(Timestamp);
            bytes[4] = bytesTimeStamp[3];
            bytes[5] = bytesTimeStamp[2];
            bytes[6] = bytesTimeStamp[1];
            bytes[7] = bytesTimeStamp[0];

            //Byte from 8 to 11
            Byte[] bytesSourceId = BitConverter.GetBytes(SourceId);
            bytes[8] = bytesSourceId[3];
            bytes[9] = bytesSourceId[2];
            bytes[10] = bytesSourceId[1];
            bytes[11] = bytesSourceId[0];

            //Data
            Array.Copy(PayloadBytesBuffer, 0, bytes, HeaderSize, PayloadBytesBuffer.Length);

            //Fertig
            return bytes;
        }

        public override string ToString()
        {
            return string.Format(" RtpPacket V:{0}, P:{1}, EX:{2}, CSRC:{3}, M:{4}, PT:{5}, SQ:{6}, TS:{7}, SSRC:{8}", Version, Padding, Extension, CsrcCount, Marker, PayloadType, SequenceNumber, Timestamp, SourceId);
                
        }
    }
}
